@@ -29,7 +29,7 @@ import socket
 import hashlib
 
 def _encrypt_password(password):
-    return hashlib.md5(password.encode()).hexdigest()
+    return hashlib.md5(password.encode()).hexdigest() if password is not None else None
 
 class _Method(object):
     def __init__(self, call, name, username=None, password=None):
@@ -60,7 +60,7 @@ class _Method(object):
             request['password'] = _encrypt_password(self._password)
 
         if request['method'] == '__getfile__':
-            return self.call(json.dumps(request))
+            return self.call(json.dumps(request), generator=True)
 
         resp = self.call(json.dumps(request))
         if resp is None:
@@ -165,14 +165,24 @@ class ServerProxy(object):
         self._username = username
         self._password = password
 
-    def _request(self, request):
+    def _response_generator(self, response):
+        while True:
+            data = response.read(262144) # 262144 == 256k
+            if not data:
+                break
+            yield data
+
+    def _request(self, request, generator=False):
         # call a method on the remote server
         try:
             response = self.__transport.request(request)
         except socket.error:
             raise NetworkSocketError
         if response.status == 200:
-            return response.read()
+            if generator:
+                return self._response_generator(response)
+            else:
+                return response.read()
         elif response.status == 400:
             raise BadRequestError(response.read().decode())
         elif response.status == 401:
@@ -261,6 +271,10 @@ class BatchCall(object):
 
         response = json.loads(self._server._request(req).decode())
 
+        # catch error in auth
+        if type(response) == dict and response['error'] is not None:
+            raise RemoteException(response['error']['error'])  ## btc fixme - also pass exception type (e.g. IntegrityError)
+
         result = []
         for i,r in enumerate(response):
             if r['id'] != self._queue[i].id:
